/**
  Example program to demonstrate when signal handlers are needed.
  Orion Sky Lawlor, olawlor@acm.org, 2006/02/03 (Public Domain)
*/
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

/** In a real program, these would really access the 
    hardware to turn the power on and off.  Imagine it's
    very dangerous to exit the program without turning the
    hardware off (like a rocket engine, a welding robot, etc.).
*/
void turn_on_hardware(void) {
	printf("Turning on hardware...\n");
}
void move_hardware_to(int position) {
	printf("  Moving hardware to position %d\n",position);
}
void turn_off_hardware(void) {
	printf("Turning off hardware...\n");
}
void run_hardware(void) {
	turn_on_hardware();
	/* Imagine this is a really complicated piece of code
	   that figures out how to move the hardware... */
	for (int step=0;step<20;step++) {
		printf("Step %d\n",step);
		int biasfactor=step-10; /* compensates for hardware bias */
		int position=step+4/biasfactor;
		move_hardware_to(position);
	}
	turn_off_hardware();
}

void signal_handler(int sig) { /* <- Called when a SIGFPE occurs */
	printf("HIT SIGNAL!  Exiting!\n");
	turn_off_hardware();
	exit(1);
}

#ifdef _WIN32 /* Windows: avoid "Program performed an illegal operation" dialog */
#include <windows.h>
extern "C"
LONG WINAPI bad(LPEXCEPTION_POINTERS p) {
	printf("Hit unhandledExceptionFilter\n");
	return EXCEPTION_EXECUTE_HANDLER; /* call signal handler */
}
void win_init(void) {
	SetUnhandledExceptionFilter((LPTOP_LEVEL_EXCEPTION_FILTER)bad);
}
#else /* not windows: no special setup is needed */
void win_init(void) {}
#endif

volatile double f=0.0f; /* enable floating-point code for windows */

int main() {
	win_init();

	signal(SIGFPE,signal_handler); /* <- Listen for signal */
	
	run_hardware();
}
